#! /usr/bin/env perl

##
### watch.pl -- watch static websites for changes
##
### Author: Sascha Lüdecke <sascha@meta-x.de>
##
### Copyright (C) 2000 Moxon AKA Sascha Lüdecke
##
##  This program is free software; you can redistribute it and/or modify
##  it under the terms of the GNU General Public License as published by
##  the Free Software Foundation; either version 2 of the License, or
##  (at your option) any later version.
##
##  This program is distributed in the hope that it will be useful,
##  but WITHOUT ANY WARRANTY; without even the implied warranty of
##  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##  GNU General Public License for more details.
##
##  You should have received a copy of the GNU General Public License
##  along with this program; if not, write to the Free Software
##  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
##
###
## $Id: watch.pl,v 1.7 2002/06/11 14:56:13 saschal Exp $


# This script check adresses given as URL for changes.  It will report status
# of all known files on stdout. 
#
# commands: (see printhelp for options)
# readsum -- Calculate the md5-sums of all known urls
# check   -- check known urls for change

local ($w3c, $lynx, $w3m);

###########################################################################
###########################################################################
##
## configuration part (please adopt pathnames to your system)
##

## How to fetch files ?
##
$w3c  = "w3c -timeout 10 2>/dev/null";
$w3m  = "w3m -dump_source";
$lynx = "lynx -source";

## other programs needed
$md5sum = "md5sum";

## location of some files
##
$sums = "$ENV{HOME}/.wwwwatch.sums";
$urls = "$ENV{HOME}/.wwwwatch";

##
## your work is done here.
##
###########################################################################
###########################################################################

#
###########################################################################
#
# Default settings

$browser = $w3m;

#
###########################################################################
#
# Commandline processing
#

if (@ARGV == 0) { &printhelp; die "\nNo parameters given !!" }

while ($_ = shift(@ARGV)) {
    /--browser=(.*)/ && do { 
	$browser = $$1;
	unless (defined $browser) {
	    print "Browser `$1' not supported yet. (try --help)\n";
	    exit 1;
	}
	next;
    };
#     /-lynx/ && do { $browser = $lynx; next };
#     /-w3c/ && do { $browser = $w3c; next };
    /--debug/ && do { $debug = 1; next };
    /--h/ && do { &printhelp; exit 0};
    /readsum/ && do { &readsum; exit 0 };
    /check/ && do { &check; exit 0 };
    print "Unknown paramter: ", $_, "\n";
}

print "no command specified (see --help).\n";

exit 2;

#
###########################################################################
#

# ---------------------------------------------------------------------------
#
# printhelp
#
# say some words about options and invocation
#
# ---------------------------------------------------------------------------

sub printhelp () {
    print << "END"
$0 [params] command

Commands:
    readsum       : initialize list of checksums
    check         : check pages for changes

Parameter:

    --browser=PRG : Choose browser to fetch the pages.  Legal values are:
                    lynx, w3c, w3m
    --debug       : more output to follow program run
    --help        : this text
END

}

# ---------------------------------------------------------------------------
#
# readsum
#
# Calculate the md5-sums of the files given in $urls.  They will be saved in
# $sums
#
# ---------------------------------------------------------------------------

sub readsum () {
    unlink $sums;
    open URLS, $urls;
    while (<URLS>) {
	chop;
	if (length($_) > 0) {
	    @_ = split;
	    print STDERR "Getting " . $_[0] . "\n" if $debug;
	    $_ = quotemeta $_[0];
	    system("$browser $_ | $md5sum "
		   . "| sed -e 's/-\$/\t$_/' >>$sums");
	}
    }
    close URLS;
}

# ---------------------------------------------------------------------------
#
# check
#
# check status of urls in $urls
#
# ---------------------------------------------------------------------------

sub check () {
    my (%sumnew, %sumold, 
	%changed, %removed, %new, 
	$key);
    local %urldesc;

    # read old sums
    # hash: url -> md5
    if (-s $sums) {
	open OLD, $sums;
	while (<OLD>) {
	    chop;
	    @_ = split;
	    $sumold{$_[1]} = $_[0];
	}
	close OLD;
    } 

    # calculate new md5-sums
    unlink $sums;
    readsum;
    open NEW, $sums;
    while (<NEW>) {
	chop;
	@_ = split;
	$sumnew{$_[1]} = $_[0]; 
    }
    close NEW;

    # Compare the sums
    # url changed, unchanged, removed, new
    #
    # changed/unchanged or new will be substracted from $sumold
    # so removed urls remain there.
    #
    foreach $key (keys %sumnew) {
	if (exists $sumold{$key}) {
	    if ($sumnew{$key} eq $sumold{$key}) {
		$unchanged{$key} = $sumnew{$key};
	    } else {
		$changed{$key} = $sumnew{$key}
	    }
	    delete $sumold{$key}
	} else {
	    $new{$key} = $sumnew{$key}
	}
    }
    # removed urls are left in $sumold
    %removed = %sumold;

    # read description of urls
    open URLS, $urls;
    while (<URLS>) {
	chop;
	if (length($_) > 0) {
	    if (/^(\S*)\s+(.*)$/) {
		$urldesc{$1} = $2;
	    };
	}
    }
    close URLS;

    #
    # print stati
    #

    # changed, new and removed
    prefprint("NEW", %new);
    prefprint("CHG", %changed);
    prefprint("REM", %removed);

    # unchanged urls
    foreach $key (sort keys %unchanged) {
	if (exists $urldesc{$key}) {
	    print "=  : " . $urldesc{$key};
	} else {
	    print  "=  : " . $key;
	}
	print "\n";
	
    }

}


# ---------------------------------------------------------------------------
#
# prefprint
#
# prints all urls in the given hashtable with given prefix. global %urldesc
# will be used to print descriptions of urls
#
# ---------------------------------------------------------------------------

sub prefprint($%) {
    my ($prefix, %urls) = @_;
    
    foreach $key (sort keys %urls) {
	print "$prefix: ";
	print $urldesc{$key} . "\n   : " if exists $urldesc{$key};
	print $key . "\n";
    }
    
}


# Fin
